`timescale 1ns/1ps

module keypad(rst, clk, keypad_data, keypad_scan, real_key_data);

input rst;    
input clk;
input [3:0] keypad_data;
output reg [2:0] keypad_scan;
output reg [7:0] real_key_data;

reg [2:0] tmp_keypad_scan;

wire key_stop;
reg [3:0] key_data;

assign key_stop = keypad_data[3] | keypad_data[2] | keypad_data[1] | keypad_data[0];

always@(posedge(clk)) 
begin
    if (rst==1)
       begin
           tmp_keypad_scan <= 3'b000;
           keypad_scan <= 3'b000;
       end    
    else begin
       if(key_stop==1)
           $display("stop ");
       else begin
          case(tmp_keypad_scan)
              3'b000 : begin
                       keypad_scan <= 3'b001;
                       tmp_keypad_scan <= 3'b001;
                       end
              3'b001 : begin
                       keypad_scan <= 3'b010;
                       tmp_keypad_scan <= 3'b010;
                       end
              3'b010 : begin
                       keypad_scan <= 3'b100;
                       tmp_keypad_scan <= 3'b100;
                       end
              3'b100 : begin
                       keypad_scan <= 3'b001;
                       tmp_keypad_scan <= 3'b001;
                       end
           endcase
          end
        end
end


always@(posedge(clk)) 
begin
    case(tmp_keypad_scan)
        3'b001 : 
        begin
           case(keypad_data) 
              4'b0001 : key_data <=1;
              4'b0010 : key_data <=4;
              4'b0100 : key_data <=7;
              4'b1000 : key_data <=0;
              default : key_data <=0;
           endcase
        end
        3'b010 : 
        begin
           case(keypad_data) 
              4'b0001 : key_data <=2;
              4'b0010 : key_data <=5;
              4'b0100 : key_data <=8;
              4'b1000 : key_data <=0;
              default : key_data <=0;
           endcase
            
        end
        3'b100 : 
        begin
           case(keypad_data) 
              4'b0001 : key_data <=3;
              4'b0010 : key_data <=6;
              4'b0100 : key_data <=9;
              4'b1000 : key_data <=0;
              default : key_data <=0;
           endcase
            
        end
        default : 
           key_data <= 0;
   endcase
end
            
always@(posedge(clk)) 
begin
    case (key_data) 
        0 : real_key_data <= 8'b00000000;
        1 : real_key_data <= 8'b00000001;
        2 : real_key_data <= 8'b00000010;
        3 : real_key_data <= 8'b00000100;
        4 : real_key_data <= 8'b00001000;
        5 : real_key_data <= 8'b00010000;
        6 : real_key_data <= 8'b00100000;
        7 : real_key_data <= 8'b01000000;
        8 : real_key_data <= 8'b10000000;
        default : real_key_data <= 8'b00000000;
    endcase        
end

endmodule